//-----------------------------------------------------------------------
// File : LaplacianFilter.hlsl
// Desc : Laplacian Filtering
// Copyright(c) Project Asura. All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2011/08/20 [Pocol] VK쐬
// <End>

//-----------------------------------------------------------------------
// Constant Buffer and Global Variables
//-----------------------------------------------------------------------
Texture2D    ColorTex : register( t0 );
SamplerState ColorSmp : register( s0 );

/////////////////////////////////////////////////////////////////////////
// VSInput structure
/////////////////////////////////////////////////////////////////////////
struct VSInput
{
	float3 Position : POSITION;
	float2 TexCoord : TEXCOORD;
};

/////////////////////////////////////////////////////////////////////////
// VSOutput structure
/////////////////////////////////////////////////////////////////////////
struct VSOutput
{
	float4 Position : SV_POSITION;
	float2 TexCoord : TEXCOORD0;
};

//----------------------------------------------------------------------
//! @brief _VF[_Gg[|Cg
//----------------------------------------------------------------------
VSOutput VSFunc( VSInput input )
{
	VSOutput output = (VSOutput)0;

	output.Position = float4( input.Position, 1.0f );
	output.TexCoord = input.TexCoord;

	return output;
}


//----------------------------------------------------------------------
//! @brief sNZVF[_Gg[|Cg
//----------------------------------------------------------------------
float4 PSFunc( VSOutput input ) : SV_TARGET
{
	//4ߖTvVAtB^
    float invW = 0.0f;
    float invH = 0.0f;
    {
        float w = 0.0f;
        float h = 0.0f;
        ColorTex.GetDimensions( w, h );
        invW = 1.0f / w;
        invH = 1.0f / h;
    }
    float4 c0 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  0, invH *  0 ) ); //i
    float4 c1 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  0, invH * -1 ) ); //i
    float4 c2 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  0, invH *  1 ) ); //i
    float4 c3 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW * -1, invH *  0 ) ); //i
    float4 c4 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  1, invH *  0 ) ); //iE
    float4 c5 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW * -1, invH * -1 ) ); //i
    float4 c6 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  1, invH * -1 ) ); //iE
    float4 c7 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW * -1, invH *  1 ) ); //i
    float4 c8 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  1, invH *  1 ) ); //iE

    float4 output = c4 + c3 + c2 + c1 + (c0 * -4.0);
	return output;
}